import os
from PyQt4 import QtCore, QtGui, uic

(Ui_Brightness, BrightnessBaseClass) = uic.loadUiType("core/ui/brightness.ui")

class ScreenButton(QtGui.QPushButton):
  def __init__(self, parent):
    QtGui.QPushButton.__init__(self, parent)

    self.desktop=self.parent().parent()

    self.setFocusPolicy(QtCore.Qt.NoFocus)
    self.setFlat(1)

    self.setMenu(QtGui.QMenu(self))
    QtCore.QObject.connect(self.menu(), QtCore.SIGNAL("aboutToShow()"), self.buildDynamicMenu)
    QtCore.QObject.connect(self.menu(), QtCore.SIGNAL("triggered(QAction*)"), self.actionSelected)

    m=self.menu()

    sm=QtGui.QMenu("Rotation", m)
    sm.setIcon(QtGui.QIcon("core/icons/randr.png"))
    self.rotation_menu=sm

    sm=QtGui.QMenu("Virtual Size", m)
    sm.setIcon(QtGui.QIcon("core/icons/window_fullscreen.png"))
    self.virtual_size_menu=sm

    sm=QtGui.QMenu("Brightness", m)
    sm.setIcon(QtGui.QIcon("core/icons/energy_star.png"))
    self.brightness_menu=sm
    
    m.addMenu(self.rotation_menu)
    m.addMenu(self.virtual_size_menu)
    m.addMenu(self.brightness_menu)

    self.setIcon(QtGui.QIcon("core/icons/terminal.png"))

  def buildDynamicMenu(self):
    """Dynamically build the screen menu."""
    self.buildRotationMenu()
    self.buildVirtualSizeMenu()
    self.buildBrightnessMenu()

  def buildRotationMenu(self):
    """Dynamically build the rotation menu."""
    sm=self.rotation_menu
    sm.clear()
    
    rotations_possible=None
    current_rotation=None
    for l in os.popen("xrandr --query").read().split("\n"):
      if l[:18]=="Rotations possible":
        rotations_possible=l[21:].split()
      elif l[:16]=="Current rotation":
        current_rotation=l[19:]

    if not rotations_possible or len(rotations_possible)==1:
      sm.setEnabled(0)
    else:
      for r in rotations_possible:
        action=QtGui.QAction(r[0].upper()+r[1:], sm)
        action.setCheckable(1)
        action.rotation=r
        if r==current_rotation:
          action.setChecked(1)
        sm.addAction(action)
      sm.setEnabled(1)

  def buildBrightnessMenu(self):
    """Dynamically build the brightness menu."""
    sm=self.brightness_menu
    sm.clear()

    if self.desktop.device.hasScreenBrightnessControl():
      brightness=self.desktop.device.getScreenBrightness()
      action=QtGui.QAction("Level (%d)..." % brightness, sm)
      action.brightness=brightness
      sm.addAction(action)

    if self.desktop.device.hasScreenPowerControl():
      p=self.desktop.device.getScreenPower()
      if p: state="off"
      else: state="on"
      action=QtGui.QAction("Switch "+state+" the screen", sm)
      action.power=1-p
      sm.addAction(action)
    
  def buildVirtualSizeMenu(self):
    """Dynamically build the virtual size menu."""
    sm=self.virtual_size_menu
    sm.clear()
    
    current_size=self.desktop.getCurrentWidgetVirtualSize()
    for s in ( ("Auto", None), 
               ("500x500", QtCore.QSize(500, 500)), 
               ("1000x1000", QtCore.QSize(1000, 1000)),
               ("1500x1500", QtCore.QSize(1500, 1500)) ):
      action=QtGui.QAction(s[0], sm)
      action.setCheckable(1)
      action.virtual_size=s[1]
      if action.virtual_size==current_size:
        action.setChecked(1)
      sm.addAction(action)

  def actionSelected(self, action):
    """Handle the selection of a menu item."""
    if hasattr(action, "rotation"):
      os.system("xrandr -o "+action.rotation)
    elif hasattr(action, "virtual_size"):
      self.desktop.setCurrentWidgetVirtualSize(action.virtual_size)
    elif hasattr(action, "brightness"):
      d=BrightnessDialog(self.desktop.device, action.brightness)
      if d.exec_()==QtGui.QDialog.Rejected:
        self.desktop.device.setScreenBrightness(action.brightness)
    elif hasattr(action, "power"):
      self.desktop.device.setScreenPower(action.power)


class BrightnessDialog(BrightnessBaseClass, Ui_Brightness):
  def __init__(self, device, brightness):
    BrightnessBaseClass.__init__(self)
    self.setupUi(self)
    self.device=device
    self.brightness_slider.setValue(brightness)
    QtCore.QObject.connect(self.brightness_slider, QtCore.SIGNAL("valueChanged(int)"), self.changeBrightness)

  def changeBrightness(self, num):
    self.device.setScreenBrightness(num)
