#!/bin/sh

# script for guided release
if [ -z "$1" ]; then 
    echo "usage : $0 <version>"
    echo "        where version is something like '0.1'"
    exit 1
fi
version=$1


# check the working directory is trunk
[ -f auimd.py ] && [ -d ../trunk ] || { echo "$0 must be run from trunk"; exit 1; }

# make sure everything is up to date and clean
res=`svn up .`
[ $? != 0 ] && { echo "Error while updating"; exit 1; }
case $res in
    "At revision"*)
        ;;

    *)
        echo "Something has been modified, check updates and try again :"
        echo $res
        exit 1
        ;;
esac

# check that there is no pending modifications
res=`svn st -q .`
[ $? != 0 ] && { echo "Error while querying status"; exit 1; }
if [ "$res" ]; then
    echo "Some modification may be pending, check it and try again"
    echo $res
    exit 1
fi

# generate the changelog
svn2cl.sh --group-by-day --authors=AUTHORS

echo -n "ChangeLog generated, "
answer=
while [ "$answer" != "ok" ];  do
    echo "check it and type 'ok' :"
    read answer
done

svn ci ChangeLog -m "Updated ChangeLog"
[ $? != 0 ] && { echo "Error while commiting ChangeLog"; exit 1; }

svn cp ../trunk ../tags/auimd-$version
[ $? != 0 ] && { echo "Error while copying trunk to tag"; exit 1; }

echo "Tag prepared, about to commit, "
answer=
while [ "$answer" != "ok" ];  do
    echo "check it and type 'ok' :"
    read answer
done

svn ci ../tags/auimd-$version -m "Release AUIMD version $version"
[ $? != 0 ] && { echo "Arrgg ! Error while commiting tag !"; exit 1; }
echo
echo "That's all folks !"
